<?php
/*
 * Plugin Name: SideMenu Premium
 * Version: 1.2.5
 * Plugin URI: https://webd.uk/product/sidemenu-upgrade/
 * Description: Adds even more options to the SideMenu
 * Author: Webd Ltd
 * Author URI: https://webd.uk
 */



if (!defined('ABSPATH')) {
    exit('This isn\'t the page you\'re looking for. Move along, move along.');
}



if (!class_exists('sidemenu_premium_class')) {

	class sidemenu_premium_class {

        public static $version = '1.2.5';

		function __construct() {

            add_action('init', array($this, 'sidemenu_init'));
            add_action('wp_footer', array($this, 'sidemenu_wp_footer'));
            add_action('customize_register', array($this, 'sidemenu_customize_register'), 999);
            add_action('wp_head' , array($this, 'sidemenu_header_output'), 11);

            if (is_admin()) {

                add_action('load-post.php', array($this, 'sidemenu_theme_options_metabox_setup'));
                add_action('load-post-new.php', array($this, 'sidemenu_theme_options_metabox_setup'));
                add_filter('pre_set_site_transient_update_plugins', 'sidemenuPremium::check_plugin_update');
                add_filter('plugins_api', 'sidemenuPremium::get_plugin_information', 10, 3);
                add_filter('after_plugin_row_' . plugin_basename(__FILE__), 'sidemenuPremium::activate_purchase_js', 10, 3);
                add_action('wp_ajax_sidemenu_activate_purchase', 'sidemenuPremium::activate_purchase');
                add_action('admin_notices', 'sidemenuPremium::upgrade_notice');
                add_filter('upgrader_pre_download', 'sidemenuPremium::upgrader_pre_download', 10, 3);
                add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'sidemenu_add_plugin_action_links'));
                add_filter('plugin_row_meta', 'sidemenuPremium::plugin_row_meta', 10, 4);
                add_action('customize_controls_enqueue_scripts', 'sidemenuCommon::enqueue_customize_controls_js');
                add_action('customize_controls_enqueue_scripts', array($this, 'sidemenu_enqueue_customize_controls_js'));

            } else {

                add_action('customize_preview_init', array($this, 'sidemenu_enqueue_customize_preview_js'));
                add_action('wp_enqueue_scripts', array($this, 'sidemenu_enqueue_scripts'));

            }

            add_action('customize_register', 'webd_customize_register');

		}

		function sidemenu_add_plugin_action_links($links) {

			$settings_links = sidemenuCommon::plugin_action_links(add_query_arg('autofocus[section]', 'sidemenu', admin_url('customize.php')), true);

			return array_merge($settings_links, $links);

		}

        function sidemenu_customize_register($wp_customize) {

            if (!class_exists('reset_customizer_class')) {

                $wp_customize->add_section('reset_customizer', array(
                    'title'     => __('Reset Customizer', 'reset-customizer'),
                    'description'  => __('Reset Customizer allows you to reset the options within each section of the customizer to their defaults without affecting other settings. You can also backup, delete and restore entire sets of customizer theme mods.', 'sidemenu'),
                    'priority'     => 0
                ));

                sidemenuCommon::add_hidden_control($wp_customize, 'install_reset_customizer', 'reset_customizer', '', sprintf(wp_kses(__('<a href="%s" class="button-primary">Install Reset Customizer plugin</a>.', 'sidemenu'), array('a' => array('href' => array(), 'class' => array()))), esc_url(wp_nonce_url(add_query_arg(array('action' => 'install-plugin', 'plugin' => 'reset-customizer'), self_admin_url('update.php')), 'install-plugin_reset-customizer'))));

            }



            $upgrade_link = '<a href="' . sidemenuCommon::upgrade_link() . '" title="' . __('Upgrade to SideMenu Premium', 'sidemenu') . '">' . __('Upgrade to SideMenu Premium', 'sidemenu') . '</a>';
            $upgrade_nag = $upgrade_link . __(' to activate this option.', 'sidemenu');



            $dashicons = array();

            if (file_exists(ABSPATH . WPINC . '/css/dashicons.css')) {

                $dashicons_css = explode("\n", implode('', file(ABSPATH . WPINC . '/css/dashicons.css')));

                foreach ($dashicons_css as $css_entry) {

                    if (strlen($css_entry) > 10 && substr($css_entry, 0, 11) === '.dashicons-') {

                        $dashicon_class = sanitize_key(preg_replace(array('/^.dashicons-/', '/:before {$/'), '', $css_entry));
                        $dashicons[$dashicon_class] = __(ucwords(str_replace('-', ' ', $dashicon_class)), 'sidemenu');

                    }

                }

                asort($dashicons);

            }

            if ($dashicons) {

                $control_label = __('Dashicon', 'sidemenu');
                $control_description = sprintf(wp_kses(__('Choose your own <a href="%s">dashicon</a> for the button that opens the SideMenu.', 'sidemenu'), array('a' => array('href' => array()))), esc_url('https://developer.wordpress.org/resource/dashicons/'));

                if (sidemenuPremium::request_permission()) {

                    $wp_customize->add_setting('sidemenu_dashicon', array(
                        'default'       => 'menu-alt',
                        'type'          => 'theme_mod',
                        'transport'     => 'refresh',
                        'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                    ));
                    $wp_customize->add_control('sidemenu_dashicon', array(
                        'label'         => $control_label,
                        'description'   => $control_description,
                        'section'       => 'sidemenu_open_button',
                        'settings'      => 'sidemenu_dashicon',
                        'type'          => 'select',
                        'choices'       => $dashicons
                    ));

                } else {

                    sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_dashicon', 'sidemenu_open_button', $label = $control_label, $control_description . ' ' . $upgrade_nag);

                }

            }

            $control_label = __('Dashicon Size', 'sidemenu');
            $control_description = __('Set the size of the dashicon.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_dashicon_size', array(
                    'default'       => '20',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_dashicon_size', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_open_button',
                    'settings'      => 'sidemenu_dashicon_size',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 10,
                        'max'   => 100,
                        'step'  => 1
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_dashicon_size', 'sidemenu_open_button', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Transition Duration', 'sidemenu');
            $control_description = __('Set how long it takes for the SideMenu to open.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_transition_duration', array(
                    'default'       => '3',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_transition_duration', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_transition_duration',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 1,
                        'max'   => 11,
                        'step'  => 1
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_transition_duration', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Transition Effect', 'sidemenu');
            $control_description = __('Choose the animaation effect when opening the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_transition_effect', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                ));
                $wp_customize->add_control('sidemenu_transition_effect', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_transition_effect',
                    'type'          => 'select',
                    'choices'       => array(
                        'ease' => __('ease', 'sidemenu'),
                        'linear' => __('linear', 'sidemenu'),
                        'ease-in' => __('ease-in', 'sidemenu'),
                        'ease-out' => __('ease-out', 'sidemenu'),
                        '' => __('ease-in-out (Default)', 'sidemenu')
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_transition_effect', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Auto-Open', 'sidemenu');
            $control_description = __('Set which pages the SideMenu should open on automatically.', 'sidemenu');

            if (sidemenuPremium::request_permission() == true) {

                $auto_open_locations = array(
                    'all' => __('Everywhere', 'sidemenu'),
                    'front_page' => __('Front Page', 'sidemenu'),
                    'home' => __('Main Blog Page', 'sidemenu'),
                    'single' => __('Single Posts', 'sidemenu'),
                    'archive' => __('Archive Pages', 'sidemenu')
                );

                foreach(get_post_types(array('public' => 'true')) as $post_type) {

                    $post_type_object = get_post_type_object($post_type);
                    $auto_open_locations['post_type/' . $post_type] = __('Post Type: ', 'sidemenu') . $post_type_object->label;

                    foreach (get_object_taxonomies($post_type, 'objects') as $taxonomy) {

                        $auto_open_locations['taxonomy/' . $post_type . '/' . $taxonomy->name] = $post_type_object->label . __(' Taxonomy: ', 'sidemenu') . $taxonomy->label;

                    }

                }

                $wp_customize->add_setting('sidemenu_auto_open', array(
                    'default'       => array(
    				    'author',
    				    'post-date',
    				    'comments',
    				    'sticky'
    				),
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_multiple_options'
                ));
                $wp_customize->add_control(new webd_Customize_Control_Checkbox_Multiple($wp_customize, 'sidemenu_auto_open', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_auto_open',
                    'settings'      => 'sidemenu_auto_open',
                    'choices'       => $auto_open_locations
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_auto_open', 'sidemenu_auto_open', $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Auto-Open Delay', 'sidemenu');
            $control_description = __('Set the delay before the SideMenu opens on posts and pages set to auto-open the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_auto_open_delay', array(
                    'default'       => '0',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_auto_open_delay', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_auto_open',
                    'settings'      => 'sidemenu_auto_open_delay',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 0,
                        'max'   => 5000,
                        'step'  => 100
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_auto_open_delay', 'sidemenu_auto_open', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Overlay Color', 'sidemenu');
            $control_description = __('Set the background color of the overlay that dims the main website.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_overlay_color', array(
                    'default'       => '#000000',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color'
                ));
                $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'sidemenu_overlay_color', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                	'settings'      => 'sidemenu_overlay_color'
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_overlay_color', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Overlay Opacity', 'sidemenu');
            $control_description = __('Set the opacity of the overlay that dims the main website.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_overlay_opacity', array(
                    'default'       => '7',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_overlay_opacity', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                    'settings'      => 'sidemenu_overlay_opacity',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 1,
                        'max'   => 11,
                        'step'  => 1
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_overlay_opacity', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Desktop SideMenu Width', 'sidemenu');
            $control_description = __('Set the width of the SideMenu as a percentage of the display width on larger screens.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_width', array(
                    'default'       => '45',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_width', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_width',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 5,
                        'max'   => 100,
                        'step'  => 5
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_width', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Desktop SideMenu Minimum Width', 'sidemenu');
            $control_description = __('Set the minimum width of the SideMenu on larger screens.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_min_width', array(
                    'default'       => 0,
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_min_width', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_min_width',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 0,
                        'max'   => 640,
                        'step'  => 10
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_min_width', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Mobile SideMenu Width', 'sidemenu');
            $control_description = __('Set the width of the SideMenu as a percentage of the display width on smaller screens.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_width_mobile', array(
                    'default'       => '100',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_width_mobile', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_width_mobile',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 5,
                        'max'   => 100,
                        'step'  => 5
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_width_mobile', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Mobile SideMenu Minimum Width', 'sidemenu');
            $control_description = __('Set the minimum width of the SideMenu on smaller screens.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_min_width_mobile', array(
                    'default'       => 0,
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_min_width_mobile', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_min_width_mobile',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 0,
                        'max'   => 320,
                        'step'  => 10
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_min_width_mobile', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Rounded Corners', 'sidemenu');
            $control_description = __('Add rounded corners to the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_rounded_corners', array(
                    'default'       => '0',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_rounded_corners', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_rounded_corners',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 0,
                        'max'   => 40,
                        'step'  => 1
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_rounded_corners', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Height', 'sidemenu');
            $control_description = __('Set the height of the SideMenu as a percentage of the display height.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_height', array(
                    'default'       => '100',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_height', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_height',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 5,
                        'max'   => 100,
                        'step'  => 5
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_height', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Grow SideMenu Content', 'sidemenu');
            $control_description = __('Spreads out the content of the SideMenu so that it fills the available height.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_grow_content', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_boolean'
                ));
                $wp_customize->add_control('sidemenu_grow_content', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_grow_content',
                    'type'          => 'checkbox'
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_grow_content', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('No Content Animation', 'sidemenu');
            $control_description = __('Prevents the site content from moving when the SideMenu opens.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_no_content_animation', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_boolean'
                ));
                $wp_customize->add_control('sidemenu_no_content_animation', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_no_content_animation',
                    'type'          => 'checkbox'
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_no_content_animation', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Prevent Scroll Close', 'sidemenu');
            $control_description = __('Prevents user scrolling from closing the sidemenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_prevent_scroll_close', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_boolean'
                ));
                $wp_customize->add_control('sidemenu_prevent_scroll_close', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_general',
                    'settings'      => 'sidemenu_prevent_scroll_close',
                    'type'          => 'checkbox'
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_prevent_scroll_close', 'sidemenu_general', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Background Color', 'sidemenu');
            $control_description = __('Set the background color of the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_background_color', array(
                    'default'       => '#666666',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color'
                ));
                $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'sidemenu_background_color', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                	'settings'      => 'sidemenu_background_color'
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_background_color', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Hide Close Button', 'sidemenu');
            $control_description = __('Do not show the SideMenu close button.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_hide_close', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_boolean'
                ));
                $wp_customize->add_control('sidemenu_hide_close', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_close_button',
                    'settings'      => 'sidemenu_hide_close',
                    'type'          => 'checkbox'
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_hide_close', 'sidemenu_close_button', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            if ($dashicons) {

                $control_label = __('Close Dashicon', 'sidemenu');
                $control_description = sprintf(wp_kses(__('Choose your own <a href="%s">dashicon</a> for the button that closes the SideMenu.', 'sidemenu'), array('a' => array('href' => array()))), esc_url('https://developer.wordpress.org/resource/dashicons/'));

                if (sidemenuPremium::request_permission()) {

                    $wp_customize->add_setting('sidemenu_close_dashicon', array(
                        'default'       => 'no-alt',
                        'type'          => 'theme_mod',
                        'transport'     => 'refresh',
                        'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                    ));
                    $wp_customize->add_control('sidemenu_close_dashicon', array(
                        'label'         => $control_label,
                        'description'   => $control_description,
                        'section'       => 'sidemenu_close_button',
                        'settings'      => 'sidemenu_close_dashicon',
                        'type'          => 'select',
                        'choices'       => $dashicons
                    ));

                } else {

                    sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_close_dashicon', 'sidemenu_close_button', $label = $control_label, $control_description . ' ' . $upgrade_nag);

                }

            }

            $control_label = __('Rounded Corner', 'sidemenu');
            $control_description = __('Add a rounded corner to the SideMenu close icon (make sure you give it a background color).', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_close_rounded_corner', array(
                    'default'       => '0',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_close_rounded_corner', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_close_button',
                    'settings'      => 'sidemenu_close_rounded_corner',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 0,
                        'max'   => 40,
                        'step'  => 1
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_close_rounded_corner', 'sidemenu_close_button', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Close Icon Color', 'sidemenu');
            $control_description = __('Set the color of the close icon in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_close_icon_color', array(
                    'default'       => '#ffffff',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color'
                ));
                $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'sidemenu_close_icon_color', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                	'settings'      => 'sidemenu_close_icon_color'
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_close_icon_color', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Close Icon Background Color', 'sidemenu');
            $control_description = __('Set the color of the close icon background in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_close_icon_background_color', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color'
                ));
                $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'sidemenu_close_icon_background_color', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                	'settings'      => 'sidemenu_close_icon_background_color'
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_close_icon_background_color', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Title Alignment', 'sidemenu');
            $control_description = __('Align the titles in the SideMenu to the left, center or right.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_title_text_align', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                ));
                $wp_customize->add_control('sidemenu_title_text_align', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_title_text_align',
                    'type'          => 'select',
                    'choices'       => array(
                        '' => __('Left', 'sidemenu'),
                        'center' => __('Center', 'sidemenu'),
                        'right' => __('Right', 'sidemenu')
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_title_text_align', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Title Font Case', 'sidemenu');
            $control_description = __('Change the font case of the titles in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_title_text_transform', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                ));
                $wp_customize->add_control('sidemenu_title_text_transform', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_title_text_transform',
                    'type'          => 'select',
                    'choices'       => array(
                        '' => __('None', 'sidemenu'),
                        'capitalize' => __('Capitalise', 'sidemenu'),
                        'uppercase' => __('Uppercase', 'sidemenu'),
                        'lowercase' => __('Lowercase', 'sidemenu')
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_title_text_transform', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Title Font Size', 'sidemenu');
            $control_description = __('Change the font size of the titles in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_title_font_size', array(
                    'default'       => '15',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_title_font_size', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_title_font_size',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 5,
                        'max'   => 30,
                        'step'  => 1
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_title_font_size', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Title Font Weight', 'sidemenu');
            $control_description = __('Change the font weight of the titles in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_title_font_weight', array(
                    'default'       => '700',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_title_font_weight', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_title_font_weight',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 100,
                        'max'   => 900,
                        'step'  => 100
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_title_font_weight', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Title Color', 'sidemenu');
            $control_description = __('Set the color of titles in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_title_color', array(
                    'default'       => '#ffffff',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color'
                ));
                $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'sidemenu_title_color', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                	'settings'      => 'sidemenu_title_color'
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_title_color', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Menu Padding', 'sidemenu');
            $control_description = __('Set the padding inbetween menu items in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_menu_padding', array(
                    'default'       => 0,
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_menu_padding', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_menu_padding',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 0,
                        'max'   => 10,
                        'step'  => 1
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_menu_padding', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            if ($dashicons) {

                $control_label = __('Sub-Menu Toggle Dashicon', 'sidemenu');
                $control_description = sprintf(wp_kses(__('Choose your own <a href="%s">dashicon</a> for the sub-menu toggle button.', 'sidemenu'), array('a' => array('href' => array()))), esc_url('https://developer.wordpress.org/resource/dashicons/'));

                if (sidemenuPremium::request_permission()) {

                    $wp_customize->add_setting('sidemenu_submenu_toggle', array(
                        'default'       => 'arrow-right-alt2',
                        'type'          => 'theme_mod',
                        'transport'     => 'refresh',
                        'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                    ));
                    $wp_customize->add_control('sidemenu_submenu_toggle', array(
                        'label'         => $control_label,
                        'description'   => $control_description,
                        'section'       => 'sidemenu_submenus',
                        'settings'      => 'sidemenu_submenu_toggle',
                        'type'          => 'select',
                        'choices'       => $dashicons
                    ));

                } else {

                    sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_submenu_toggle', 'sidemenu_submenus', $label = $control_label, $control_description . ' ' . $upgrade_nag);

                }

            }

            $control_label = __('Sub-Menu Toggle Rotation', 'sidemenu');
            $control_description = __('Choose how the sub-menu toggle should rotate on click.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_sub_menu_toggle_rotate', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                ));
                $wp_customize->add_control('sidemenu_sub_menu_toggle_rotate', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_submenus',
                    'settings'      => 'sidemenu_sub_menu_toggle_rotate',
                    'type'          => 'select',
                    'choices'       => array(
                        '-360' => __('-360&deg; (Default)', 'sidemenu'),
                        '-270' => __('-270&deg; (Default)', 'sidemenu'),
                        '-180' => __('-180&deg; (Default)', 'sidemenu'),
                        '-90' => __('-90&deg; (Default)', 'sidemenu'),
                        '0' => __('No Rotation', 'sidemenu'),
                        '' => __('90&deg; (Default)', 'sidemenu'),
                        '180' => __('180&deg; (Default)', 'sidemenu'),
                        '270' => __('270&deg; (Default)', 'sidemenu'),
                        '360' => __('360&deg; (Default)', 'sidemenu')
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_sub_menu_toggle_rotate', 'sidemenu_submenus', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Smaller Sub-Menu Items', 'sidemenu');
            $control_description = __('Shows sub-menu items with a smaller font size than their parent\'s.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_smaller_submenu', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_boolean'
                ));
                $wp_customize->add_control('sidemenu_smaller_submenu', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_submenus',
                    'settings'      => 'sidemenu_smaller_submenu',
                    'type'          => 'checkbox'
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_smaller_submenu', 'sidemenu_submenus', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Text Alignment', 'sidemenu');
            $control_description = __('Align the text in the SideMenu to the left, center or right.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_text_align', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                ));
                $wp_customize->add_control('sidemenu_text_align', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_text_align',
                    'type'          => 'select',
                    'choices'       => array(
                        '' => __('Left', 'sidemenu'),
                        'center' => __('Center', 'sidemenu'),
                        'right' => __('Right', 'sidemenu')
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_text_align', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Text Font Case', 'sidemenu');
            $control_description = __('Change the font case of the text in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_text_transform', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                ));
                $wp_customize->add_control('sidemenu_text_transform', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_text_transform',
                    'type'          => 'select',
                    'choices'       => array(
                        '' => __('None', 'sidemenu'),
                        'capitalize' => __('Capitalise', 'sidemenu'),
                        'uppercase' => __('Uppercase', 'sidemenu'),
                        'lowercase' => __('Lowercase', 'sidemenu')
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_text_transform', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Text Font Size', 'sidemenu');
            $control_description = __('Change the font size of the text in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_text_font_size', array(
                    'default'       => '10',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_text_font_size', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_text_font_size',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 5,
                        'max'   => 20,
                        'step'  => 1
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_text_font_size', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Text Color', 'sidemenu');
            $control_description = __('Set the color of text in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_text_color', array(
                    'default'       => '#ffffff',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color'
                ));
                $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'sidemenu_text_color', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                	'settings'      => 'sidemenu_text_color'
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_text_color', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Link Color', 'sidemenu');
            $control_description = __('Set the color of links in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_link_color', array(
                    'default'       => '#d3d3d3',
                    'type'          => 'theme_mod',
                    'transport'     => 'postMessage',
                    'sanitize_callback' => 'sanitize_hex_color'
                ));
                $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'sidemenu_link_color', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                	'settings'      => 'sidemenu_link_color'
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_link_color', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('SideMenu Link Hover Color', 'sidemenu');
            $control_description = __('Set the color of hovered links in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_link_hover_color', array(
                    'default'       => '#000000',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sanitize_hex_color'
                ));
                $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'sidemenu_link_hover_color', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                	'settings'      => 'sidemenu_link_hover_color'
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_link_hover_color', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Section Border Width', 'sidemenu');
            $control_description = __('Set the width of the border inbetween sections in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_section_border_width', array(
                    'default'       => '2',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'absint'
                ));
                $wp_customize->add_control('sidemenu_section_border_width', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_section_border_width',
                    'type'          => 'range',
                    'input_attrs' => array(
                        'min'   => 1,
                        'max'   => 11,
                        'step'  => 1
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_section_border_width', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Section Border Color', 'sidemenu');
            $control_description = __('Set the color of the border inbetween sections in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_section_border_color', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sanitize_hex_color'
                ));
                $wp_customize->add_control(new WP_Customize_Color_Control($wp_customize, 'sidemenu_section_border_color', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_colors',
                	'settings'      => 'sidemenu_section_border_color'
                )));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_section_border_color', 'sidemenu_colors', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Section Border Style', 'sidemenu');
            $control_description = __('Set the style of the border inbetween sections in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_section_border_style', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_options'
                ));
                $wp_customize->add_control('sidemenu_section_border_style', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_style',
                    'settings'      => 'sidemenu_section_border_style',
                    'type'          => 'select',
                    'choices'       => array(
                        'dotted' => __('Dotted', 'sidemenu'),
                        'dashed' => __('Dashed', 'sidemenu'),
                        '' => __('Solid', 'sidemenu'),
                        'double' => __('Double', 'sidemenu'),
                        'groove' => __('3D Groove', 'sidemenu'),
                        'ridge' => __('3D Ridge', 'sidemenu'),
                        'inset' => __('3D Inset', 'sidemenu'),
                        'outset' => __('3D Outset', 'sidemenu')
                    )
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_section_border_style', 'sidemenu_style', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

            $control_label = __('Menu Item Hover Animation', 'sidemenu');
            $control_description = __('Adds a hover animation underneath each menu item in the SideMenu.', 'sidemenu');

            if (sidemenuPremium::request_permission()) {

                $wp_customize->add_setting('sidemenu_menu_item_hover_anim', array(
                    'default'       => '',
                    'type'          => 'theme_mod',
                    'transport'     => 'refresh',
                    'sanitize_callback' => 'sidemenuCommon::sanitize_boolean'
                ));
                $wp_customize->add_control('sidemenu_menu_item_hover_anim', array(
                    'label'         => $control_label,
                    'description'   => $control_description,
                    'section'       => 'sidemenu_menu_items',
                    'settings'      => 'sidemenu_menu_item_hover_anim',
                    'type'          => 'checkbox'
                ));

            } else {

                sidemenuCommon::add_hidden_control($wp_customize, 'sidemenu_menu_item_hover_anim', 'sidemenu_menu_items', $label = $control_label, $control_description . ' ' . $upgrade_nag);

            }

        }

        function sidemenu_header_output() {

?>
<!--Customizer CSS-->
<style type="text/css">
<?php

if (sidemenuPremium::request_permission()) {

            $mod = absint(get_theme_mod('sidemenu_overlay_opacity'));

            if ($mod) {

                $mod = ($mod - 1) / 10;

?>
.sidemenu_open>.cover {
    opacity: <?php echo $mod; ?>;
}
<?php

            }

            $sidemenu_width = get_theme_mod('sidemenu_width');

            if (absint($sidemenu_width)) {

                if (get_theme_mod('sidemenu_position')) {

?>
@media screen and (min-width: 48em) {
    .sidemenu {
        left: -<?php echo absint($sidemenu_width); ?>%;
        width: <?php echo absint($sidemenu_width); ?>%;
    }
    .sidemenu_open>div {
        transform: translateX(<?php echo absint($sidemenu_width / 2); ?>%);
    }
}
<?php

                } else {

?>
@media screen and (min-width: 48em) {
    .sidemenu {
        right: -<?php echo absint($sidemenu_width); ?>%;
        width: <?php echo absint($sidemenu_width); ?>%;
    }
    .sidemenu_open>div {
        transform: translateX(-<?php echo absint($sidemenu_width / 2); ?>%);
    }
}
<?php

                }

            }

            $mod = get_theme_mod('sidemenu_min_width');

            if (absint($mod)) {

?>
@media screen and (min-width: 48em) {
    .sidemenu {
        min-width: <?php echo absint($mod); ?>px;
    }
}
<?php

                if (!absint($sidemenu_width)) {

                    $sidemenu_width = 45;

                }

?>
@media screen and (min-width: 48em) and (max-width: <?php echo ceil((100 * absint($mod)) / absint($sidemenu_width)); ?>px) {
    .sidemenu {
<?php

                if (get_theme_mod('sidemenu_position')) {

?>
        left: -<?php echo absint($mod); ?>px;
<?php

                } else {

?>
        right: -<?php echo absint($mod); ?>px;
<?php

                }

?>
    }
}
<?php

            }

            $sidemenu_width = get_theme_mod('sidemenu_width_mobile');

            if (absint($sidemenu_width)) {

                if (get_theme_mod('sidemenu_position')) {

?>
@media screen and (max-width: 48em) {
    .sidemenu {
        left: -<?php echo absint($sidemenu_width); ?>%;
        width: <?php echo absint($sidemenu_width); ?>%;
    }
    .sidemenu_open>div {
        transform: translateX(<?php echo absint($sidemenu_width / 2); ?>%);
    }
}
<?php

                } else {

?>
@media screen and (max-width: 48em) {
    .sidemenu {
        right: -<?php echo absint($sidemenu_width); ?>%;
        width: <?php echo absint($sidemenu_width); ?>%;
    }
    .sidemenu_open>div {
        transform: translateX(-<?php echo absint($sidemenu_width / 2); ?>%);
    }
}
<?php

                }

            }

            $mod = get_theme_mod('sidemenu_min_width_mobile');

            if (absint($mod)) {

?>
@media screen and (max-width: 48em) {
    .sidemenu {
        min-width: <?php echo absint($mod); ?>px;
    }
}
<?php

                if (!absint($sidemenu_width)) {

                    $sidemenu_width = 100;

                }

?>
@media screen and (max-width: <?php echo ceil((100 * absint($mod)) / absint($sidemenu_width)); ?>px) {
    .sidemenu {
<?php

                if (get_theme_mod('sidemenu_position')) {

?>
        left: -<?php echo absint($mod); ?>px;
<?php

                } else {

?>
        right: -<?php echo absint($mod); ?>px;
<?php

                }

?>
    }
}
<?php

            }

            $mod = absint(get_theme_mod('sidemenu_height'));

            if ($mod) {

?>
@media screen and (min-width: 48em) {
    .sidemenu {
        top: <?php echo (100 - $mod) / 2; ?>%;
        bottom: <?php echo (100 - $mod) / 2; ?>%;
    }
}
<?php

            }

            if (get_theme_mod('sidemenu_grow_content')) {

?>
.sidemenu {
	display: flex;
	flex-direction: column;
}
.sidemenu>section {
	flex: 1;
}
.sidemenu>section:first-child, .sidemenu>section:last-child {
	flex-grow: 0;
}
<?php

            }

            sidemenuCommon::generate_css('.sidemenu_open>div', 'transform', 'sidemenu_no_content_animation', '', '', 'none');
            sidemenuCommon::generate_css('.open_sidemenu .dashicons', 'width', 'sidemenu_dashicon_size', '', 'px');
            sidemenuCommon::generate_css('.open_sidemenu .dashicons', 'height', 'sidemenu_dashicon_size', '', 'px');
            sidemenuCommon::generate_css('.open_sidemenu .dashicons', 'font-size', 'sidemenu_dashicon_size', '', 'px');

            $duration = absint(get_theme_mod('sidemenu_transition_duration'));
            $effect = get_theme_mod('sidemenu_transition_effect');

            if ($duration || $effect) {

                $duration = ($duration - 1) / 10;
                if (!$effect) { $effect = 'ease-in-out'; }

?>
.sidemenu {
    transition: left <?php echo $duration; ?>s <?php echo $effect; ?>, right <?php echo $duration; ?>s <?php echo $effect; ?>, opacity <?php echo $duration; ?>s <?php echo $effect; ?>;
}
body>.cover {
    transition: opacity <?php echo $duration; ?>s <?php echo $effect; ?>;
}
<?php

            }

            sidemenuCommon::generate_css('body>.cover', 'background-color', 'sidemenu_overlay_color');
            sidemenuCommon::generate_css('.sidemenu', 'background-color', 'sidemenu_background_color');
            sidemenuCommon::generate_css('.sidemenu section:nth-child(2)', 'margin-top', 'sidemenu_hide_close', '', '', '0');
            sidemenuCommon::generate_css('.sidemenu .close_sidemenu', 'color', 'sidemenu_close_icon_color');
            sidemenuCommon::generate_css('.sidemenu .close_sidemenu', 'background-color', 'sidemenu_close_icon_background_color');
            sidemenuCommon::generate_css('.sidemenu section h2', 'text-align', 'sidemenu_title_text_align');
            sidemenuCommon::generate_css('.sidemenu section h2', 'text-transform', 'sidemenu_title_text_transform');
            sidemenuCommon::generate_css('.sidemenu section h2', 'font-size', 'sidemenu_title_font_size', '', 'em', (get_theme_mod('sidemenu_title_font_size') ? (absint(get_theme_mod('sidemenu_title_font_size')) / 10) : ''));
            sidemenuCommon::generate_css('.sidemenu section h2', 'font-weight', 'sidemenu_title_font_weight');
            sidemenuCommon::generate_css('.sidemenu section h2', 'color', 'sidemenu_title_color');

            $mod = absint(get_theme_mod('sidemenu_menu_padding'));

            if ($mod) {

?>
.sidemenu .widget li, .sidemenu .menu li {
padding-top: <?php echo $mod / 10; ?>em;
padding-bottom: <?php echo $mod / 10; ?>em;
}
<?php

            }

            sidemenuCommon::generate_css('.sidemenu .submenu-toggle.open', 'transform', 'sidemenu_sub_menu_toggle_rotate', 'rotate(', 'deg)');
            sidemenuCommon::generate_css('.sidemenu .widget li li, .sidemenu .menu li li', 'font-size', 'sidemenu_smaller_submenu', '', '', 'smaller');
            sidemenuCommon::generate_css('.sidemenu p, .sidemenu li', 'text-align', 'sidemenu_text_align');
            sidemenuCommon::generate_css('.sidemenu p, .sidemenu li', 'text-transform', 'sidemenu_text_transform');
            sidemenuCommon::generate_css('.sidemenu p, .sidemenu li', 'font-size', 'sidemenu_text_font_size', '', 'em', (get_theme_mod('sidemenu_text_font_size') ? (absint(get_theme_mod('sidemenu_text_font_size')) / 10) : ''));
            sidemenuCommon::generate_css('.sidemenu', 'color', 'sidemenu_text_color');
            sidemenuCommon::generate_css('.sidemenu section a', 'color', 'sidemenu_link_color');
            sidemenuCommon::generate_css('.sidemenu section a:hover, .sidemenu section ul li a:hover', 'color', 'sidemenu_link_hover_color');
            sidemenuCommon::generate_css('.sidemenu section', 'border-bottom-width', 'sidemenu_section_border_width', '', 'px', (get_theme_mod('sidemenu_section_border_width') ? (absint(get_theme_mod('sidemenu_section_border_width')) - 1) : ''));
            sidemenuCommon::generate_css('.sidemenu section', 'border-bottom-color', 'sidemenu_section_border_color');
            sidemenuCommon::generate_css('.sidemenu section', 'border-bottom-style', 'sidemenu_section_border_style');

            if (get_theme_mod('sidemenu_menu_item_hover_anim')) {

                $mod_color = get_theme_mod('sidemenu_menu_item_border_color');
                if (!$mod_color) { $mod_color = 'white'; }

                $mod_hover_color = get_theme_mod('sidemenu_menu_item_border_hover_color');
                if (!$mod_hover_color) { $mod_hover_color = 'black'; }

?>
.widget_nav_menu a, .sidemenu-navigation a {
    position: relative;
    padding-bottom: 0.5rem;
}
.widget_nav_menu a:before,  .sidemenu-navigation a:before {
    content: "";
    position: absolute;
    width: 0;
    height: 2px;
    bottom: 0;
    left: 0;
    background-color: <?php echo $mod_color; ?>;
    visibility: hidden;
    transition: all 0.3s ease-in-out;
}
.widget_nav_menu a:hover:before,  .sidemenu-navigation a:hover:before {
    visibility: visible;
    width: 100%;
    background-color: <?php echo $mod_hover_color; ?>;
}
<?php

            }

            $mod = absint(get_theme_mod('sidemenu_rounded_corners'));

            if ($mod) {

                if (get_theme_mod('sidemenu_position')) {

?>
.sidemenu {
	border-radius: 0 <?php echo $mod; ?>px <?php echo $mod; ?>px 0;
}
<?php

                } else {

?>
.sidemenu {
	border-radius: <?php echo $mod; ?>px 0 0 <?php echo $mod; ?>px;
}
<?php

                }

            }

            $mod = absint(get_theme_mod('sidemenu_close_rounded_corner'));

            if ($mod) {

?>
.sidemenu .close_sidemenu {
	border-radius: 0 0 0 <?php echo $mod; ?>px;
}
<?php

            }

}

?>
</style> 
<!--/Customizer CSS-->
<?php

        }

        function sidemenu_enqueue_customize_preview_js() {

            wp_enqueue_script('sidemenu-premium-customize-preview', plugin_dir_url(__FILE__) . 'js/customize-preview.js', array('jquery', 'customize-preview'), sidemenuPremium::plugin_version(), true);

        }

        function sidemenu_enqueue_customize_controls_js() {

            wp_enqueue_script('sidemenu-premium-customize-controls', plugin_dir_url(__FILE__) . 'js/customize-controls.js', array('jquery', 'customize-controls'), sidemenuPremium::plugin_version(), true);

        }

        function sidemenu_theme_options_metabox_setup() {

            add_action('add_meta_boxes', array($this, 'sidemenu_add_theme_options_metabox'));

            if (sidemenuPremium::request_permission()) {

                add_action('save_post', array($this, 'sidemenu_save_theme_options_meta'), 10, 2);

            }

        }

        function sidemenu_add_theme_options_metabox() {

            add_meta_box('sidemenu_meta_box', __('SideMenu', 'sidemenu'), array($this, 'sidemenu_render_theme_options_metabox'), array('post', 'page'), 'side');

        }

        function sidemenu_render_theme_options_metabox($post) {

            $upgrade_link = '<a href="' . sidemenuCommon::upgrade_link() . '" title="' . __('Upgrade SideMenu', 'sidemenu') . '">' . __('Upgrade SideMenu', 'sidemenu') . '</a>';

            if (get_option('sidemenu_purchased') == false) {

                if (get_option('sidemenu_trial_date') && time() < (strtotime('+1 week', get_option('sidemenu_trial_date')))) {

                    $expiring_in = ceil(abs((strtotime('+1 week', get_option('sidemenu_trial_date'))) - time())/60/60/24);
                    $expiring_text = '<p><span class="attention">' . sprintf(_n('SideMenu Premium plugin trial expires in less than %s day!', 'SideMenu Premium plugin trial expires in less than %s days!', $expiring_in, 'sidemenu'), $expiring_in) . '</span>';
                    echo('<strong>' . $expiring_text . '</strong>' . ' ' . $upgrade_link . ' ' . __('to keep using all the options after that time.', 'sidemenu') . '</p>');

                }

            }

            wp_nonce_field('sidemenu', 'sidemenu-meta-nonce');

            if (sidemenuPremium::request_permission()) {

                $auto_open_args = get_theme_mod('sidemenu_auto_open');
                $auto_open = false;

                if (is_array($auto_open_args) && (
                    in_array('all', $auto_open_args) ||
                    (in_array('front_page', $auto_open_args) && get_option('page_on_front') == $post->ID) ||
                    (in_array('home', $auto_open_args) && get_option('page_for_posts') == $post->ID) ||
                    (in_array('single', $auto_open_args) && $post->post_type == 'page') ||
                    (in_array('post_type/' . $post->post_type, $auto_open_args))
                )) {

                    $auto_open = true;

                }

?>
<p><input type="checkbox" name="sidemenu-auto-open" id="sidemenu-auto-open" value="1" <?php checked(get_post_meta($post->ID, 'sidemenu_auto_open', true), '1' ); ?> />
<label for="sidemenu-auto-open"><?php echo $auto_open ? __('Do not auto-open SideMenu', 'sidemenu') : __('Auto-open SideMenu', 'sidemenu'); ?></label></p><?php

                $custom_sidemenus = get_posts(array(
                    'numberposts' => -1,
                    'post_type' => 'sidemenu'
                ));

                if ($custom_sidemenus) {

?>
<p><label for="sidemenu-custom-sidemenu"><?php echo __('Replace SideMenu content with:', 'sidemenu'); ?></label>
<select name="sidemenu-custom-sidemenu" id="sidemenu-custom-sidemenu">
<option value=""<?php selected(absint(get_post_meta($post->ID, 'sidemenu_custom_sidemenu', true)), 0); ?>>Select ...</option>
<?php

                    foreach ($custom_sidemenus as $custom_sidemenu) {

?>
<option value="<?php echo $custom_sidemenu->ID; ?>"<?php selected(absint(get_post_meta($post->ID, 'sidemenu_custom_sidemenu', true)), $custom_sidemenu->ID); ?>><?php echo esc_html($custom_sidemenu->post_title); ?></option>
<?php

                    }

?>
</select></p>
<?php

                }

            } else {

?>

<p><?php echo $upgrade_link; ?> <?php _e('to choose where to auto-open the SideMenu or to entirely replace the SideMenu content for specific pages and posts.', 'sidemenu'); ?></p>

<?php
            }

        }

        function sidemenu_save_theme_options_meta($post_id, $post) {

            if (!isset($_POST['sidemenu-meta-nonce']) || !wp_verify_nonce($_POST['sidemenu-meta-nonce'], 'sidemenu')) {

                return;

            }

            $post_type = get_post_type_object($post->post_type);

            if (!current_user_can($post_type->cap->edit_post, $post_id)) {

                return;

            }

            if (isset($_POST['sidemenu-auto-open']) && $_POST['sidemenu-auto-open'] == '1') {

                update_post_meta($post_id, 'sidemenu_auto_open', 1);

            } else {

                delete_post_meta($post_id, 'sidemenu_auto_open');

            }

            if (isset($_POST['sidemenu-custom-sidemenu']) && absint($_POST['sidemenu-custom-sidemenu'])) {

                update_post_meta($post_id, 'sidemenu_custom_sidemenu', absint($_POST['sidemenu-custom-sidemenu']));

            } else {

                delete_post_meta($post_id, 'sidemenu_custom_sidemenu');

            }

        }

        function sidemenu_enqueue_scripts() {

            global $post;

            $auto_open_args = get_theme_mod('sidemenu_auto_open');

            if (!is_admin() && (is_array($auto_open_args) || ((is_singular()) && '1' === get_post_meta($post->ID, 'sidemenu_auto_open', true)))) {

                if (is_archive()) {

                    global $wp_query;
                    $taxonomy = $wp_query->get_queried_object();

                }

                $auto_open = false;

                if (
                    is_array($auto_open_args) &&
                        (in_array('all', $auto_open_args) ||
                        (in_array('front_page', $auto_open_args) && is_front_page()) ||
                        (in_array('home', $auto_open_args) && is_home()) ||
                        (in_array('single', $auto_open_args) && is_single()) ||
                        (in_array('archive', $auto_open_args) && is_archive()) ||
                        (is_single() && in_array('post_type/' . get_post_type() , $auto_open_args)) ||
                        (is_page() && in_array('post_type/page', $auto_open_args)) ||
                        (is_archive() && isset($taxonomy->taxonomy) && in_array('taxonomy/' . get_post_type() . '/' . $taxonomy->taxonomy, $auto_open_args))
                    )
                ) {

                    $auto_open = true;

                }

                if ((is_singular()) && '1' === get_post_meta($post->ID, 'sidemenu_auto_open', true)) {

                    $auto_open = !$auto_open;

                }

                if ($auto_open) {

                		wp_enqueue_script('sidemenu-premium-js', plugin_dir_url(__FILE__) . 'js/sidemenu-premium.js', array(), sidemenuPremium::plugin_version(), true);

                        $l10n = array();
                        $l10n['autoopen'] = true;
                        $l10n['autoopendelay'] = get_theme_mod('sidemenu_auto_open_delay');

                        wp_localize_script(
                            'sidemenu-premium-js',
                            'sideMenuPremium',
                            $l10n
                        );

                }

            }

        }

        public function sidemenu_init() {

        	$labels = [
        		"name" => __( "SideMenus", "twentytwenty" ),
        		"singular_name" => __( "SideMenu", "twentytwenty" ),
        		"menu_name" => __( "SideMenus", "twentytwenty" ),
        		"all_items" => __( "SideMenus", "twentytwenty" ),
        		"add_new" => __( "Add New", "twentytwenty" ),
        		"add_new_item" => __( "Add new SideMenu", "twentytwenty" ),
        		"edit_item" => __( "Edit SideMenu", "twentytwenty" ),
        		"new_item" => __( "New SideMenu", "twentytwenty" ),
        		"view_item" => __( "View SideMenu", "twentytwenty" ),
        		"view_items" => __( "View SideMenus", "twentytwenty" ),
        		"search_items" => __( "Search SideMenus", "twentytwenty" ),
        		"not_found" => __( "No SideMenus found", "twentytwenty" ),
        		"not_found_in_trash" => __( "No SideMenus found in bin", "twentytwenty" ),
        		"parent" => __( "Parent SideMenu:", "twentytwenty" ),
        		"featured_image" => __( "Featured image for this SideMenu", "twentytwenty" ),
        		"set_featured_image" => __( "Set featured image for this SideMenu", "twentytwenty" ),
        		"remove_featured_image" => __( "Remove featured image for this SideMenu", "twentytwenty" ),
        		"use_featured_image" => __( "Use as featured image for this SideMenu", "twentytwenty" ),
        		"archives" => __( "SideMenu archives", "twentytwenty" ),
        		"insert_into_item" => __( "Insert into SideMenu", "twentytwenty" ),
        		"uploaded_to_this_item" => __( "Upload to this SideMenu", "twentytwenty" ),
        		"filter_items_list" => __( "Filter SideMenus list", "twentytwenty" ),
        		"items_list_navigation" => __( "SideMenus list navigation", "twentytwenty" ),
        		"items_list" => __( "SideMenus list", "twentytwenty" ),
        		"attributes" => __( "SideMenus attributes", "twentytwenty" ),
        		"name_admin_bar" => __( "SideMenu", "twentytwenty" ),
        		"item_published" => __( "SideMenu published", "twentytwenty" ),
        		"item_published_privately" => __( "SideMenu published privately.", "twentytwenty" ),
        		"item_reverted_to_draft" => __( "SideMenu reverted to draft.", "twentytwenty" ),
        		"item_scheduled" => __( "SideMenu scheduled", "twentytwenty" ),
        		"item_updated" => __( "SideMenu updated.", "twentytwenty" ),
        		"parent_item_colon" => __( "Parent SideMenu:", "twentytwenty" ),
        	];

        	$args = [
        		"label" => __( "SideMenus", "twentytwenty" ),
        		"labels" => $labels,
        		"description" => "Completely replace the SideMenu content on specific pages or posts with this content. Choose which SideMenu is shown on a specific page or post by choosing it in the page / post editor.",
        		"public" => true,
        		"publicly_queryable" => false,
        		"show_ui" => true,
        		"show_in_rest" => true,
        		"rest_base" => "",
        		"rest_controller_class" => "WP_REST_Posts_Controller",
        		"has_archive" => false,
        		"show_in_menu" => true,
        		"show_in_nav_menus" => false,
        		"delete_with_user" => false,
        		"exclude_from_search" => true,
        		"capability_type" => "post",
        		"map_meta_cap" => true,
        		"hierarchical" => false,
        		"rewrite" => false,
        		"query_var" => false,
        		"menu_position" => 20,
        		"menu_icon" => "dashicons-menu",
        		"supports" => [ "title", "editor" ],
        		"show_in_graphql" => false,
        	];

        	register_post_type( "sidemenu", $args );
        }

        public function sidemenu_wp_footer() {

            if (sidemenuPremium::request_permission() && is_singular()) {

                $custom_sidemenu = absint(get_post_meta(get_the_ID(), 'sidemenu_custom_sidemenu', true));

                if ($custom_sidemenu) {

                    $custom_sidemenu = get_post($custom_sidemenu);

                    if ($custom_sidemenu) {

?>
<section id="custom-sidemenu"><?php echo apply_filters('the_content', $custom_sidemenu->post_content); ?></section>
<script type="text/javascript">
    Array.prototype.forEach.call(document.querySelectorAll('.sidemenu>section'), function(sidemenuContent) {
        sidemenuContent.remove();
    });  
        document.getElementsByClassName('sidemenu')[0].appendChild(document.getElementById('custom-sidemenu'));
</script>
<?php

                    }

                }

            }

        }

	}

    if (!class_exists('sidemenuPremium')) {

        require_once(dirname(__FILE__) . '/includes/class-sidemenu-premium.php');

    }

    function sidemenu_premium_init() {

        if (class_exists('sidemenu_class') && class_exists('sidemenuCommon')) {

            sidemenuPremium::start_trial();
            $sidemenu_premium_object = new sidemenu_premium_class();

        } else {

            add_action('admin_notices', 'sidemenuPremium::free_plugin_notice');

        }

    }

    add_action('plugins_loaded', 'sidemenu_premium_init');

}

?>
